<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Teknisa\Model\Gestaopessoas\Entities\GpeSolicitacao;

class ProcessamentoSolicitacaoFactory {
    
    const TP_SOLI_FERIAS            = 1;
    const TP_SOLI_MOVIM             = 2;
    const TP_SOLI_SALARIO           = 3;
    const TP_SOLI_TREINAMENTO       = 4;
    const TP_SOLI_DESLIGAMENTO      = 5;
    const TP_SOLI_PESSOAL           = 6;
    const TP_SOLI_ALTERADADOS       = 7;
    const TP_SOLI_ALTERAENDERECO    = 8;
    const TP_SOLI_OCUPACAO          = 9;
    const TP_SOLI_VTRANS            = 10;
    const TP_SOLI_DEP               = 11;
    const TP_SOLI_ALTESITU          = 12;
    const TP_SOLI_PLANOSAUDE        = 13;
    const TP_SOLI_ALTERAESCALA      = 14;
    const TP_SOLI_ALTERACONTA       = 15;
    const TP_SOLI_ALTERACOMU        = 16;
    const TP_SOLI_ADMISSAO          = 17;
    const TP_SOLI_APROVAHE          = 18;
    const TP_SOLI_REEMBOLSO         = 19;
    
    /**
     * Instancia entidades de processamento das solicitacoes a partir de uma solicitacao e informacoes extras opcionais.
     * 
     * @param GpeSolicitacao  $solicitacao,
     * @param ?array          $extraData = []
     * 
     * @return ProcessamentoSolicitacao
     */
    public static function createProcessamentoSolicitacao(
        GpeSolicitacao  $solicitacao,
        ?array          $extraData = []
    ): ?ProcessamentoSolicitacao {
        switch ($solicitacao->getNrtiposolicitacao()) {
            case self::TP_SOLI_ALTERAESCALA:
                return new ProcessamentoSolicitacaoEscala($solicitacao, $extraData);
            case self::TP_SOLI_ALTERADADOS:
                return new ProcessamentoSolicitacaoDadosPessoa($solicitacao, $extraData);
            case self::TP_SOLI_OCUPACAO:
                return new ProcessamentoSolicitacaoOcupacao($solicitacao, $extraData);
            case self::TP_SOLI_SALARIO:
                return new ProcessamentoSolicitacaoSalario($solicitacao, $extraData);
            case self::TP_SOLI_DESLIGAMENTO:
                return new ProcessamentoSolicitacaoDesligamento($solicitacao, $extraData);
            case self::TP_SOLI_DEP:
                return new ProcessamentoSolicitacaoDependente($solicitacao, $extraData);
            case self::TP_SOLI_PESSOAL:
                return new ProcessamentoSolicitacaoPessoal($solicitacao, $extraData);
            case self::TP_SOLI_PLANOSAUDE:
                return new ProcessamentoSolicitacaoPlanoSaude($solicitacao, $extraData);
            case self::TP_SOLI_MOVIM:
                return new ProcessamentoSolicitacaoRemanejamento($solicitacao, $extraData);
            case self::TP_SOLI_VTRANS:
                return new ProcessamentoSolicitacaoValeTransporte($solicitacao, $extraData);
            case self::TP_SOLI_ALTERACOMU:
                return new ProcessamentoSolicitacaoFormaComunicacao($solicitacao, $extraData);
            case self::TP_SOLI_ALTERAENDERECO:
                return new ProcessamentoSolicitacaoEndereco($solicitacao, $extraData);
            case self::TP_SOLI_ALTERACONTA:
                return new ProcessamentoSolicitacaoContaCorrente($solicitacao, $extraData);
            case self::TP_SOLI_APROVAHE:
                return new ProcessamentoSolicitacaoAprovaHorasExtras($solicitacao, $extraData);
            case self::TP_SOLI_TREINAMENTO:
                return new ProcessamentoSolicitacaoTreinamento($solicitacao, $extraData);
            case self::TP_SOLI_REEMBOLSO:
                return new ProcessamentoSolicitacaoAprovaReembolso($solicitacao, $extraData);
            default:
                return null;
        }
    }
}