<?php

namespace Ems\Services;

use Ems\Repository\{ EquipeRepository, PeriodoRepository };
use Ems\Util\ArrayUtil;
use Ems\Services\EstruturaService;
use Util\Environment;
use Util\DateUtil;

class EquipeService {
    /** @var EquipeRepository */
    protected $equipeRepository;

    /** @var Environment */
    protected $environment;

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var PeriodoRepository */
    protected $periodoRepository;

    public function __construct(
        EquipeRepository $equipeRepository,
        Environment $environment,
        EstruturaService $estruturaService,
        PeriodoRepository $periodoRepository
    ) {
        $this->equipeRepository  = $equipeRepository;
        $this->environment       = $environment;
        $this->estruturaService  = $estruturaService;
        $this->periodoRepository = $periodoRepository;
    }

    public function getMonitoramentoEquipeContrato(
        int $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRESTRUTURAS' => $estruturas
        ];

        $contratos = $this->equipeRepository->getMonitoramentoEquipeContrato($params);

        return $contratos;
    }

    public function getMonitoramentoEquipeLicenca(
        int $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRESTRUTURAS' => $estruturas
        ];

        $licencas = $this->equipeRepository->getMonitoramentoEquipeLicenca($params);

        return $licencas;
    }

    public function getMonitoramentoEquipeBancoHoras(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_NRPERIODOAPURACAO'  => $nrPeriodo,
            'P_DTFINALAPURACAO'    => $dtFinalApuracao,
            'P_NRESTRUTURAS'       => $estruturas
        ];

        $saldoBancoDeHoras = $this->equipeRepository->
            getMonitoramentoEquipeBancoHoras($params);

        return $saldoBancoDeHoras;
    }

    public function getMonitoramentoEquipeAtrasos(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_NRPERIODOAPURACAO'  => $nrPeriodo,
            'P_DTFINALAPURACAO'    => $dtFinalApuracao,
            'P_NRESTRUTURAS'       => $estruturas
        ];

        $atrasos = $this->equipeRepository->
            getMonitoramentoEquipeAtrasos($params);

        return $atrasos;
    }

    public function getMonitoramentoEquipeFaltas(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_NRPERIODOAPURACAO'  => $nrPeriodo,
            'P_DTFINALAPURACAO'    => $dtFinalApuracao,
            'P_NRESTRUTURAS'       => $estruturas
        ];

        $faltas = $this->equipeRepository->
            getMonitoramentoEquipeFaltas($params);

        return $faltas;
    }

    public function getMonitoramentoEquipeHorasExtras(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_NRPERIODOAPURACAO'  => $nrPeriodo,
            'P_DTFINALAPURACAO'    => $dtFinalApuracao,
            'P_NRESTRUTURAS'       => $estruturas
        ];

        $horasExtras = $this->equipeRepository->
            getMonitoramentoEquipeHorasExtras($params);

        return $horasExtras;
    }

    public function getMonitoramentoEquipeAtestados(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtInicialApuracao = $periodo['DTINICIALAPURACAO'];
        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_DTINICIALAPURACAO'  => $dtInicialApuracao,
            'P_DTFINALAPURACAO'    => $dtFinalApuracao,
            'P_NRESTRUTURAS'       => $estruturas
        ];

        $atestados = $this->equipeRepository->
            getMonitoramentoEquipeAtestados($params);

        return $atestados;
    }

    public function getMonitoramentoEquipeFerias(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_DTFINALAPURACAO'    => $dtFinalApuracao,
            'P_NRESTRUTURAS'       => $estruturas
        ];

        $ferias = $this->equipeRepository->
            getMonitoramentoEquipeFerias($params);

        return $ferias;
    }

    private function getNrEstruturas(
        int $nrEstrutura
    ) {
       $estruturas = null;

        if ($nrEstrutura !== -1) {
            $nrEstruturas = $this->estruturaService->getTodasEstruturasInferiores(
                $nrEstrutura, true
            );

            $estruturas = ArrayUtil::toStringSQL($nrEstruturas);
        }

        return $estruturas;
    }
    
    public function getMonitoramentoEquipeAfastamentos(
        $row
    ) {
        $nrEstrutura = (int) $row['NRESTRUTURAM'];

        $nrOrg = (int) $this->environment->getNrOrg();

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'              => $nrOrg,
            'P_NRESTRUTURAS'       => $estruturas,
        ];

        $afastamentos = $this->equipeRepository->
            getMonitoramentoEquipeAfastamentos($params);

        return $afastamentos;
    }
    
    public function getMonitoramentoEquipeUltimasFerias(
        int $nrEstrutura,
        int $nrPeriodo
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
            $nrOrg, $nrPeriodo
        )[0];

        $dtFinalApuracao = $periodo['DTFINALAPURACAO'];
        $dtInicialApuracao = $periodo['DTINICIALAPURACAO'];

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_DTFINALAPURACAO'     => $dtFinalApuracao,
            'P_DTINICIALAPURACAO'   => $dtInicialApuracao,    
            'P_NRESTRUTURAS'        => $estruturas
        ];

        $ultFerias = $this->equipeRepository->
            getMonitoramentoEquipeUltimasFerias($params);

        return $ultFerias;
    }

    public function getMonitoramentoAusenciaMarcacaoPrevista(
        int $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $competencia = $this->environment->getDtmescomptencDiaCorrente();
        $diaAtual = DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_DATA);

        $estruturas = $this->getNrEstruturas($nrEstrutura);

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRESTRUTURAS' => $estruturas,
            'P_DTMESCOMPETENC' => $competencia,
            'P_DIA' => $diaAtual,
        ];

        $ausencias = $this->equipeRepository->getMonitoramentoAusenciaMarcacaoPrevista($params);

        return $ausencias;
    }
}