/* Sistemas Integrados TECNISA
   Servios

Nome: EXCLUSAO_DE_DUPLICATA

Descrio:
Gerao de boletos fictcios e excluso do boleto real de contas a receber
referentes a uma duplicata que est sendo excluda.

P_CDFILIAL    - Cdigo da Filial
P_NRLANCTODUP - Nmero da Duplicata
P_NRLANBOLREC - Nmero dos boletos - este Nr. ser incrementado dentro da procedure.
P_IDRESULT    - Identificador de resultado
                'N' - Ok.
                'D' - Existem contas (vencimentos) de duplicatas diferentes agrupadas no mesmo boleto.
                'E' - No existem contas associadas  duplicata.
                'F' - Nmero do boleto (interno) no informado.
                'I' - Boletos j baixados
                'J' - Contas ou boletos parcelados
                'L' - Conta prorrogada - (DESATIVADO)
                'M' - Duplicata associada a CNAB
Para aplicar e debugar
-- @C:\Win_Prj\Sql\Oracle\1020_FINSTP_EXCLUSAO_DE_DUPLICATA.SQL;
-- EXECUTE EXCLUSAO_DE_DUPLICATA('9999', '0000000000', '0000000000', :P_IDRESULT)
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');

Criao:
13/01/00 - Ederson

*/

CREATE OR REPLACE
PROCEDURE EXCLUSAO_DE_DUPLICATA
  (P_CDFILIAL    IN VARCHAR2,
   P_NRLANCTODUP IN VARCHAR2,
   P_NRLANBOLREC IN BOLETRECEBER.NRLANBOLREC%TYPE,
   P_IDRESULT    IN OUT VARCHAR2) IS

   V_CDFILIBOLREC CONTARECEBER.CDFILIBOLREC%TYPE;
   V_NRLANBOLREC  CONTARECEBER.NRLANBOLREC%TYPE;
   V_NRPRXLANBOL  CONTARECEBER.NRLANBOLREC%TYPE;
   V_CDCLIENTE    CLIENTE.CDCLIENTE%TYPE;
   V_NRREGFOR     NUMBER;
   V_NRBOLETO     NUMBER;
   V_PEMULTA      NUMBER;
   V_PEMORA       NUMBER;
   V_COUNT        NUMBER;
   V_CTAJBOL      NUMBER;
   V_NRBOLBAIXADO NUMBER;
   V_NRPARCELA    NUMBER;
   V_NRCNTPROR    NUMBER;
   V_CHAVE_NOTA   VARCHAR2(14);
   V_CHAVE_NOTA_AUX VARCHAR2(14);
BEGIN
   P_IDRESULT := 'N';

   /* - VERIFICA SE A DUPLICATA POSSUI CONTAS ASSOCIADAS -------------------- */
   SELECT
      COUNT(*)
   INTO
      V_NRREGFOR
   FROM
      CONTARECEBER
   WHERE
      (CDFILIALDUP = P_CDFILIAL) AND
      (NRLANCTODUP = P_NRLANCTODUP);

   IF V_NRREGFOR > 0 THEN
      FOR C_CNTBOL IN
         (SELECT
             CDFILIBOLREC,
             NRLANBOLREC
          FROM
             CONTARECEBER
          WHERE
             (CDFILIALDUP = P_CDFILIAL) AND
             (NRLANCTODUP = P_NRLANCTODUP)
         ) LOOP

         SELECT
            COUNT(*)
         INTO
            V_NRBOLETO
         FROM
            (SELECT DISTINCT
                CDFILIALDUP || NRLANCTODUP
             FROM
                CONTARECEBER
             WHERE
                (CDFILIBOLREC = C_CNTBOL.CDFILIBOLREC) AND
                (NRLANBOLREC  = C_CNTBOL.NRLANBOLREC)
             );

         /* - O Boleto possui contas associadas de duplicatas diferentes ---- */
         IF V_NRBOLETO > 1 THEN
            P_IDRESULT := 'D';
         END IF;
      END LOOP;
   ELSE
      P_IDRESULT := 'E';
   END IF;

   IF P_NRLANBOLREC = '' THEN
      P_IDRESULT := 'F';
   END IF;

   /* - VERIFICA SE O BOLETO J FOI BAIXADO ---------------------------------- */
   SELECT
      COUNT(*)
   INTO
      V_NRBOLBAIXADO
   FROM
      CONTARECEBER CR,
      BOLETRECEBER BO
   WHERE
      (CDFILIALDUP    = P_CDFILIAL)      AND
      (NRLANCTODUP    = P_NRLANCTODUP)   AND
      (BO.CDFILIAL    = CR.CDFILIBOLREC) AND
      (BO.NRLANBOLREC = CR.NRLANBOLREC)  AND
      (BO.NRBAIXAREC IS NOT NULL);

   IF V_NRBOLBAIXADO > 0 THEN
      P_IDRESULT := 'I';
   END IF;

   /* - VERIFICA SE O BOLETO DE DUPLICATA FOI PARCELAD0 ----------------------------------*/
   SELECT COUNT(DISTINCT CDFILIBOLREC||NRLANBOLREC)
     INTO V_NRPARCELA
     FROM CONTARECEBER
    WHERE CDFILIALDUP = P_CDFILIAL
      AND NRLANCTODUP = P_NRLANCTODUP;

   IF V_NRPARCELA > 1 THEN
      P_IDRESULT := 'H';
   END IF;


   /* - VERIFICA SE A CONTA FOI PRORROGADA ----------------------------------
   SELECT
      COUNT(*)
   INTO
      V_NRCNTPROR
   FROM
      CONTARECEBER
   WHERE
      (CDFILIALDUP  = P_CDFILIAL)    AND
      (NRLANCTODUP  = P_NRLANCTODUP) AND
      (DTORIGVENREC <> DTATUAVENREC)  AND
      (NRLANBOLREC IS NOT NULL);

   IF V_NRCNTPROR > 0 THEN
      P_IDRESULT := 'L';
   END IF;
   */

   -- VERIFICA SE A DUPLICATA ESTA LIGADA A UM CNABR
   SELECT
       COUNT(*)
   INTO
      V_NRCNTPROR
   FROM
      CONTARECEBER CR,
      BOLETRECEBER BO
   WHERE
      (CDFILIALDUP    = P_CDFILIAL     ) AND
      (NRLANCTODUP    = P_NRLANCTODUP  ) AND
      (BO.CDFILIAL    = CR.CDFILIBOLREC) AND
      (BO.NRLANBOLREC = CR.NRLANBOLREC ) AND
      (IDSITUCNABR <> '00')              AND
      (IDSITUCNABR IS NOT NULL);

   IF V_NRCNTPROR > 0 THEN
      P_IDRESULT := 'M';
   END IF;

   IF P_IDRESULT = 'N' THEN
      SELECT
      	MAX(CDFILIBOLREC),
      	MAX(NRLANBOLREC)
      INTO
         V_CDFILIBOLREC,
         V_NRLANBOLREC
      FROM
         CONTARECEBER
      WHERE
	      (CDFILIALDUP = P_CDFILIAL) AND
   	   (NRLANCTODUP = P_NRLANCTODUP);


      /* -- APAGA A CHAVE DA DUPLICATA ------------------------------ */
      UPDATE CONTARECEBER
      SET
         CDFILIALDUP = NULL,
         NRLANCTODUP = NULL
      WHERE
         (CDFILIALDUP = P_CDFILIAL) AND
         (NRLANCTODUP = P_NRLANCTODUP);

      V_NRPRXLANBOL := P_NRLANBOLREC;
      V_CHAVE_NOTA := ' ';
      FOR C_CNTREC IN
         (SELECT CDFILIAL,
                 NRLANCTONF,
                 NRSEQVENREC,
                 NRPARCELAREC,
                 CDTPCTRECEB
            FROM CONTARECEBER
           WHERE (CDFILIBOLREC = V_CDFILIBOLREC) AND
                 (NRLANBOLREC  = V_NRLANBOLREC)
         ORDER BY CDFILIAL, NRLANCTONF
         ) LOOP
         V_CHAVE_NOTA_AUX := C_CNTREC.CDFILIAL || C_CNTREC.NRLANCTONF;
         IF (V_CHAVE_NOTA <> V_CHAVE_NOTA_AUX) THEN
            /*VERIFICA A QUANTIDADE DE CONTAS QUE IRO SER ATUALIZADAS */
            SELECT COUNT(*)
              INTO V_COUNT
              FROM CONTARECEBER
             WHERE CDFILIAL = C_CNTREC.CDFILIAL
               AND NRLANCTONF = C_CNTREC.NRLANCTONF
               AND CDFILIBOLREC = V_CDFILIBOLREC
               AND NRLANBOLREC = V_NRLANBOLREC;
            /* -- INCLUI O NOVO BOLETO ---------------------------------------------*/
            GERA_BOLETOS(C_CNTREC.CDFILIAL,
                         C_CNTREC.NRLANCTONF,
                         V_CDFILIBOLREC,
                         V_NRPRXLANBOL,
                         C_CNTREC.CDTPCTRECEB,
                         'S',
                         V_CDFILIBOLREC,
                         V_NRLANBOLREC);

            V_NRPRXLANBOL := LPAD(TO_CHAR(TO_NUMBER(V_NRPRXLANBOL) + V_COUNT),10,'0');
            V_CHAVE_NOTA := C_CNTREC.CDFILIAL || C_CNTREC.NRLANCTONF;
         END IF;
      END LOOP;

      /* - APAGA O BOLETO, SEUS AJUSTE E O FLUXO DE CAIXA ---------------------- */
      SELECT COUNT(*)
      INTO V_CTAJBOL
      FROM AJBOLREC
      WHERE
         (CDFILIAL    = V_CDFILIBOLREC) AND
         (NRLANBOLREC = V_NRLANBOLREC);

      IF V_CTAJBOL > 0 THEN
         FOR C_AJBOL IN
            (SELECT
                MR.CDCLASSFINA,  MR.CDEMPRESA,    MR.CDFILIAL,    MR.DTMOVFLUXO,
                MR.IDMOVFLUXO,   MR.CDFILIBOLREC, MR.NRLANBOLREC, MR.CDAJUCTREC,
                MR.VRFXAJBOLREC, TA.IDAJUCTREC
             FROM
                TIPOAJRECEBER TA, MOVFXAJBOLR MR
             WHERE
                (MR.CDFILIBOLREC = V_CDFILIBOLREC) AND
                (MR.NRLANBOLREC  = V_NRLANBOLREC)  AND
                (TA.CDAJUCTREC   = MR.CDAJUCTREC)
             ) LOOP

            IF C_AJBOL.IDAJUCTREC = 'A' THEN
               ATUALIZA_FLUXO_DE_CAIXA(C_AJBOL.CDEMPRESA,
                                       C_AJBOL.CDFILIAL,
                                       C_AJBOL.DTMOVFLUXO,
                                       C_AJBOL.IDMOVFLUXO,
                                       C_AJBOL.CDCLASSFINA,
                                       C_AJBOL.VRFXAJBOLREC * -1,
                                       0);
            ELSE
               ATUALIZA_FLUXO_DE_CAIXA(C_AJBOL.CDEMPRESA,
                                       C_AJBOL.CDFILIAL,
                                       C_AJBOL.DTMOVFLUXO,
                                       C_AJBOL.IDMOVFLUXO,
                                       C_AJBOL.CDCLASSFINA,
                                       C_AJBOL.VRFXAJBOLREC,
                                       0);
            END IF;
         END LOOP;

         DELETE FROM MOVFXAJBOLR
         WHERE
            (CDFILIBOLREC = V_CDFILIBOLREC) AND
            (NRLANBOLREC  = V_NRLANBOLREC);

         DELETE FROM AJBOLREC
         WHERE
            (CDFILIAL    = V_CDFILIBOLREC) AND
            (NRLANBOLREC = V_NRLANBOLREC);
      END IF;

   	DELETE FROM BOLETRECEBER
      WHERE
         (CDFILIAL    = V_CDFILIBOLREC) AND
         (NRLANBOLREC = V_NRLANBOLREC);

      /* -- INCLUI UM BOLETO PARA CADA CONTA ------------------------- */
/*      V_NRPRXLANBOL := P_NRLANBOLREC;
      FOR C_CONTA IN
         (SELECT
             CDFILIAL,
             NRLANCTONF,
             NRSEQVENREC,
             NRPARCELAREC,
             DTORIGVENREC,
             DTATUAVENREC,
        	    CDFILIBOLREC,
        	    NRLANBOLREC,
             CDEMPRESA,
             CDFIMOVFLREC,
             DTMOVFLUXO,
             IDMOVFLUXO,
             CDCLASSFINA,
             VRORIGREC
          FROM
             CONTARECEBER
          WHERE
             (CDFILIALDUP = P_CDFILIAL) AND
             (NRLANCTODUP = P_NRLANCTODUP)
         ) LOOP

         IF C_CONTA.DTORIGVENREC <> C_CONTA.DTATUAVENREC THEN
            /* - ATUALIZA O FLUXO DE CAIXA -------------------------- */
/*            ATUALIZA_FLUXO_DE_CAIXA(C_CONTA.CDEMPRESA,
                                    C_CONTA.CDFIMOVFLREC,
                                    C_CONTA.DTMOVFLUXO,
                                    C_CONTA.IDMOVFLUXO,
                                    C_CONTA.CDCLASSFINA,
                                    C_CONTA.VRORIGREC * -1,
                                    0);

            ATUALIZA_FLUXO_DE_CAIXA(C_CONTA.CDEMPRESA,
                                    C_CONTA.CDFIMOVFLREC,
                                    C_CONTA.DTORIGVENREC,
                                    C_CONTA.IDMOVFLUXO,
                                    C_CONTA.CDCLASSFINA,
                                    C_CONTA.VRORIGREC,
                                    0);

            UPDATE CONTARECEBER
            SET
               DTATUAVENREC = C_CONTA.DTORIGVENREC,
               CDMOTIPROR   = NULL,
               DTMOVFLUXO   = C_CONTA.DTORIGVENREC
            WHERE
               (CDFILIAL     = C_CONTA.CDFILIAL)     AND
               (NRLANCTONF   = C_CONTA.NRLANCTONF)   AND
               (NRSEQVENREC  = C_CONTA.NRSEQVENREC)  AND
               (NRPARCELAREC = C_CONTA.NRPARCELAREC);

            /* -- ATUALIZA O FLUXO DE CAIXA DOS AJUSTES DA CONTA -------------*/
/*            SELECT
               COUNT(*)
            INTO
               V_NRREGFOR
            FROM
               AJCTRECEBER
            WHERE
               (CDFILIAL     = C_CONTA.CDFILIAL)    AND
               (NRLANCTONF   = C_CONTA.NRLANCTONF)  AND
               (NRSEQVENREC  = C_CONTA.NRSEQVENREC) AND
               (NRPARCELAREC = C_CONTA.NRPARCELAREC);

            IF V_NRREGFOR > 0 THEN
             	FOR C_AJUSTE IN
                  (SELECT
                      CDFILIAL,
                      NRLANCTONF,
                      NRSEQVENREC,
                      NRPARCELAREC,
                      DTAJUSTEREC,
                      CDAJUCTREC,
                      CDCLASSFINA,
                      CDEMMFAJCNTR,
                      CDFIMFAJCNTR,
                      DTMOVFLUXO,
                      IDMOVFLUXO,
                      VRAJUSTEREC
                   FROM
                      AJCTRECEBER
                   WHERE
                      (CDFILIAL     = C_CONTA.CDFILIAL)     AND
                      (NRLANCTONF   = C_CONTA.NRLANCTONF)   AND
                      (NRSEQVENREC  = C_CONTA.NRSEQVENREC)  AND
                      (NRPARCELAREC = C_CONTA.NRPARCELAREC)
                  ) LOOP

                  ATUALIZA_FLUXO_DE_CAIXA(C_AJUSTE.CDEMMFAJCNTR,
                                          C_AJUSTE.CDFIMFAJCNTR,
                                          C_AJUSTE.DTMOVFLUXO,
                                          C_AJUSTE.IDMOVFLUXO,
                                          C_AJUSTE.CDCLASSFINA,
                                          C_AJUSTE.VRAJUSTEREC * -1,
                                          0);

                  ATUALIZA_FLUXO_DE_CAIXA(C_AJUSTE.CDEMMFAJCNTR,
                                          C_AJUSTE.CDFIMFAJCNTR,
                                          C_CONTA.DTORIGVENREC,
                                          C_AJUSTE.IDMOVFLUXO,
                                          C_AJUSTE.CDCLASSFINA,
                                          C_AJUSTE.VRAJUSTEREC,
                                          0);

                  UPDATE AJCTRECEBER
                  SET DTMOVFLUXO = C_CONTA.DTORIGVENREC
                  WHERE
                     (CDFILIAL     = C_AJUSTE.CDFILIAL)     AND
                     (NRLANCTONF   = C_AJUSTE.NRLANCTONF)   AND
                     (NRSEQVENREC  = C_AJUSTE.NRSEQVENREC)  AND
                     (NRPARCELAREC = C_AJUSTE.NRPARCELAREC) AND
                     (DTAJUSTEREC  = C_AJUSTE.DTAJUSTEREC)  AND
                     (CDAJUCTREC   = C_AJUSTE.CDAJUCTREC)   AND
                     (CDCLASSFINA  = C_AJUSTE.CDCLASSFINA);
               END LOOP;
            END IF;
         END IF;


/*         V_PEMULTA := 0;
         V_PEMORA  := 0;

         SELECT
            MAX(CL.CDCLIENTE),
            MAX(CL.VRPEMULBCLI),
            MAX(CL.VRPEMORBCLI)
         INTO
            V_CDCLIENTE,
            V_PEMULTA,
            V_PEMORA
         FROM
            CONTARECEBER CR,
            NOTAFISCAL   NF,
            CLIENTE      CL
         WHERE
            (CR.CDFILIALDUP = P_CDFILIAL)    AND
            (CR.NRLANCTODUP = P_NRLANCTODUP) AND
            (NF.CDFILIAL    = CR.CDFILIAL)   AND
            (NF.NRLANCTONF  = CR.NRLANCTONF) AND
            (CL.CDCLIENTE   = NF.CDCLIENTE);

         IF (V_PEMULTA IS NULL) OR
            (V_PEMORA  IS NULL) THEN
            SELECT
               MAX(VRPEMULEMPR),
               MAX(VRPEMOREMPR)
            INTO
               V_PEMULTA,
               V_PEMORA
            FROM
               EMPRESA
            WHERE
               (CDEMPRESA = C_CONTA.CDEMPRESA);
         END IF;

         INSERT INTO BOLETRECEBER(
            CDFILIAL,
            NRLANBOLREC,
            CDCLIENTE,
            CDEMPRESA,
            VRBOLREC,
            VRPEMULBREC,
            VRPEMORBREC,
            VRFINALBREC,
            DTVENBOLREC,
            CDNOSNUMREC)
         VALUES(
            P_CDFILIAL,
            V_NRPRXLANBOL,
            V_CDCLIENTE,
            C_CONTA.CDEMPRESA,
            C_CONTA.VRORIGREC,
            V_PEMULTA,
            V_PEMORA,
            C_CONTA.VRORIGREC,
            C_CONTA.DTORIGVENREC,
            P_CDFILIAL||V_NRPRXLANBOL);

         /* -- GRAVA A CHAVE DO NOVO BOLETO NAS CONTAS DA DUPLICATA -------------*/
/*         UPDATE CONTARECEBER
         SET
            CDFILIBOLREC = P_CDFILIAL,
            NRLANBOLREC  = V_NRPRXLANBOL
         WHERE
            (CDFILIAL     = C_CONTA.CDFILIAL)     AND
            (NRLANCTONF   = C_CONTA.NRLANCTONF)   AND
            (NRSEQVENREC  = C_CONTA.NRSEQVENREC)  AND
            (NRPARCELAREC = C_CONTA.NRPARCELAREC);

         V_NRPRXLANBOL := TO_CHAR(TO_NUMBER(V_NRPRXLANBOL) + 1);
         V_NRPRXLANBOL := LPAD(V_NRPRXLANBOL, 10, '0');
      END LOOP;
 */
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('EXCLUSAO_DE_DUPLICATA','4.09.066'); END;
/
